/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "WdtLabel.h"

Wdt_LABEL * WdtLabelNew (UInt16 x, UInt16 y, UInt8 fgcolor, UInt8 bgcolor, const char *message) {
  Wdt_LABEL *p = (Wdt_LABEL *) calloc (1, sizeof (Wdt_LABEL));
  if (p) {
    p->x = x; // Upper left corner
    p->y = y;
	p->_fgcolor = fgcolor;
	p->_bgcolor = bgcolor;

    p->toredraw = true; 
    p->visibility = true; 
    p->max_width = 0;
    p->max_height = 0;
    p->message = 0; 
    p->justify = false; 
    p->current_width = 0;

    WdtLabelSetString (p, message);
  }
  return p;
}

void WdtLabelDelete (Wdt_LABEL *aLabel) {
  if (aLabel) {
    if (aLabel->message) {
      free (aLabel->message);
    }
    free (aLabel);
  }
}

void WdtLabelDraw (Wdt_LABEL *aWdtLabel, bool force) { 
  if (!aWdtLabel) {
#ifdef DEBUG
    printf ("WdtLabelDraw : WdtLabel = NULL\n");
#endif
    return;
  }
  
  if (aWdtLabel->visibility == false) return;

  if ((force == false) && (aWdtLabel->toredraw == false)) // Conditionned by toredraw flag
    return; // Nothing to do

  // Erase the largest area
  if ((aWdtLabel->max_width != 0) && (aWdtLabel->max_height != 0)) {
    if (aWdtLabel->justify == false) {
      GfxFillRectangle (aWdtLabel->x, aWdtLabel->y-aWdtLabel->max_height,
						aWdtLabel->max_width, aWdtLabel->max_height+1,
						aWdtLabel->_bgcolor);
    }
    else {
	  GfxFillRectangle (aWdtLabel->x-aWdtLabel->max_width,
						aWdtLabel->y-aWdtLabel->max_height,
						aWdtLabel->max_width,
						aWdtLabel->max_height+1,
						aWdtLabel->_bgcolor);
    }
  }
  
  if (aWdtLabel->justify == false) { // Default. X corresponds to the left part of the string
	GfxStringXY (aWdtLabel->message, 
				 aWdtLabel->x, aWdtLabel->y - FONT_HEIGHT, 
				 aWdtLabel->_fgcolor, aWdtLabel->_bgcolor);
  }
  else {
	GfxStringXY (aWdtLabel->message, 
				 aWdtLabel->x-aWdtLabel->current_width, aWdtLabel->y - FONT_HEIGHT,
				 aWdtLabel->_fgcolor, aWdtLabel->_bgcolor);
  }
  aWdtLabel->toredraw = false;
}

void WdtLabelSetString (Wdt_LABEL *aWdtLabel, const char *aString) {
  UInt16 w, h;

  if (!aWdtLabel) {
#ifdef DEBUG
    printf ("WdtLabelSetString : WdtLabel = NULL\n");
#endif
    return;
  }

  if (aWdtLabel->message) {
    if (strcmp ((const char *)aWdtLabel->message, (const char *)aString) == 0) return;
    free (aWdtLabel->message);
  }

  aWdtLabel->message = (char *) calloc (strlen ((const char *) aString)+1, sizeof (char));
  strcpy ((char *)aWdtLabel->message, (const char *)aString);

  // Compute the max_width & max_height attributes
  // to be able to erase the area
  GfxStringWidth (aString, &w, &h);
  aWdtLabel->current_width=w;

  if (aWdtLabel->max_height < h) aWdtLabel->max_height = h;
  if (aWdtLabel->max_width < w) aWdtLabel->max_width = w;

  aWdtLabel->toredraw = true; // Value has changed. Need to redraw
}

const char* WdtLabelGetString (Wdt_LABEL *aWdtLabel) {
  if (!aWdtLabel) {
#ifdef DEBUG
    printf ("WdtLabelGetString : WdtLabel = NULL\n");
#endif
    return 0;
  }

  return aWdtLabel->message;
}

bool WdtLabelGetVisibility (Wdt_LABEL *aWdtLabel) {
  if (!aWdtLabel) {
#ifdef DEBUG
    printf ("WdtLabelGetVisibility : WdtLabel = NULL\n");
#endif
    return false;
  }
  return aWdtLabel->visibility;
}

void WdtLabelSetVisibility (Wdt_LABEL *aWdtLabel, bool visibility) {
  if (!aWdtLabel) {
#ifdef DEBUG
    printf ("WdtLabelSetVisibility : WdtLabel = NULL\n");
#endif
    return; 
  }
  
  if (aWdtLabel->visibility == visibility) return; 

  aWdtLabel->visibility = visibility;
  aWdtLabel->toredraw = true; // Visible or now hidden. Need to redraw
}

void WdtLabelSetJustify (Wdt_LABEL *aWdtLabel, bool justify) {
  if (!aWdtLabel) {
#ifdef DEBUG
    printf ("WdtLabelSetJustify : WdtLabel = NULL\n");
#endif
    return;
  }

  if (aWdtLabel->justify == justify) return; // Nothing to do
  aWdtLabel->justify = justify;
  aWdtLabel->toredraw = true; 
}
// End of file
